#!/bin/bash
version=`hg tip | head -n1 | sed 's/^changeset: *//g' | sed 's/:.*$//g'`
echo "version = $version" > version.py
echo processing wxremind version $version

# from the Python Cookbook (2nd ed) recipe 16.12: "Binding Main Script and Modules into One Executable on Unix". The idea is to create a header script file, a zip file containing the *.py for the required modules and then 'cat' the header and zip files to produce a single, executable file.

# create wxremalert and wxremdata and make them executable
echo processing alert
zip ../wxremind-dist/wxremalert.zip version.py wxremindrc.py wxremalert.py 
cat wxremalert.head ../wxremind-dist/wxremalert.zip > wxremalert
chmod +x wxremalert

echo processing data
zip ../wxremind-dist/wxremdata.zip version.py wxremindrc.py wxremdata.py 
cat wxremdata.head ../wxremind-dist/wxremdata.zip > wxremdata
chmod +x wxremdata

# create wxremind and make it executable
zip ../wxremind-dist/wxremind.zip version.py wxremindrc.py wxremind.py wxremdata.py wxremalert.py wxRemAbout.py wxRemDays.py wxRemEdit.py wxRemHelp.py # wxRemHints.py 
cat wxremind.head ../wxremind-dist/wxremind.zip > wxremind
chmod +x wxremind

# create wxremhints and make it executable
echo processing wxremhints
zip ../wxremind-dist/wxremhints.zip  version.py wxremindrc.py wxRemHints.py
cat wxremhints.head ../wxremind-dist/wxremhints.zip > wxremhints
chmod +x wxremhints


# combine the sources into a compressed source file for distribution
if [ ! -d ../wxremind-dist/wxRemind-${version} ]; then
	echo "making ../wxremind-dist/wxRemind-${version}"
	mkdir ../wxremind-dist/wxRemind-${version}
else
	echo "found ../wxremind-dist/wxRemind-${version} - removing contents"
	rm -f ../wxremind-dist/wxRemind-${version}/*
fi
cp -p wxremindrc.py wxremind.py wxremind.head wxremdata.head wxremhints.head wxremdata.py wxremalert.head wxremalert.py wxRemAbout.py wxRemDays.py wxRemEdit.py wxRemHelp.py wxRemHints.py version.py make_dist.sh ../wxremind-dist/wxremind-${version}



# update CHANGES
hg log | grep -v "user" > ../wxremind-dist/docs-wxRemind/CHANGES

cd ../wxremind-dist/
tar -cvzf wxRemind-src-${version}.tgz wxremind-${version}
cd -

# combine all the executables into a compressed file for distribution
tar -cvzf ../wxremind-dist/wxRemind-${version}.tgz wxremind wxremalert wxremdata wxremhints
cd ../wxremind-dist/docs-wxRemind
