#!/usr/bin/env python
import os, os.path, wx, codecs
# from wxRemHints import Hints
from wxremindrc import *
from version import *

skelRlist = ['REM', 'ONCE']
skelMlist = ['MSG', 'MSF', 'RUN', 'CAL', 'SATISFY', 'SPECIAL', 'PS', 'PSFILE']

class TextObjectValidator(wx.PyValidator):
    """ This validator is used to ensure that the user has entered something
            into the text object editor dialog's text field.
    """
    def __init__(self, data, key):
        """ Standard constructor.
        """
        wx.PyValidator.__init__(self)
        self.data = data
        self.key = key


    def Clone(self):
        """ Standard cloner.

        """
        return TextObjectValidator(self.data, self.key)


    def Validate(self, win):
        """ Validate the contents of the given text control.
        """
        return True

    def TransferToWindow(self):
        """ Transfer data from validator to window.

        """
        textCtrl = self.GetWindow()
        if self.key in ('alert_s', 'alert_d', 'skel_r', 'skel_m', 'file_s'):
            textCtrl.SetSelection(self.data.get(self.key, 0))
        else:
            textCtrl.SetValue(self.data.get(self.key, "").encode(encoding))
            return True # Prevent wxDialog from complaining.


    def TransferFromWindow(self):
        """ Transfer data from window to validator.
        """
        textCtrl = self.GetWindow()
        if self.key in ('alert_s', 'alert_d', 'file_s'):
            self.data[self.key] = textCtrl.GetSelection()
        elif self.key in ('skel_r'):
            self.data[self.key] = skelRlist[textCtrl.GetSelection()]
        elif self.key in ('skel_m'):
            self.data[self.key] = skelMlist[textCtrl.GetSelection()]
        elif self.key in ('date', 'time', 'warn', 'msg'):
            self.data[self.key] = textCtrl.GetValue()
            if self.data[self.key] == '':
                wx.MessageBox("A required field is empty!", "Error")
                textCtrl.SetFocus()
                textCtrl.Refresh()
                return False
        else:
            self.data[self.key] = textCtrl.GetValue().encode('utf-8')
            textCtrl.Refresh()
            return True

class MyDialog(wx.Dialog):
    def __init__(self, data, type='u'):
        self.data = data
        wx.SetDefaultPyEncoding("utf-8")
        if type == 'u':
            title = "untimed event"
            about_txt = """\
This untimed event will be displayed on the event list for the 'date' set below. Click Help for details on the 'date' field and for date substitutions in the message fields.\
"""
        elif type == 't':
            title = "timed event"
            about_txt = """\
This event will be displayed on the event list for the 'date', 'time' and 'duration' set below. A visible and, optionally, an audible alert will also be internally generated at the appropriate time(s) unless the message field begins with a "%s". The type of audible alert will depend upon the configuration file settings for 'audible1', 'audible2' and 'alert_sound'. Click Help for details on the date, time and duration fields, and for date and time substitutions in the message fields.\
""" % ignore_char
        elif type == 'a':
            title = "alert event"
            about_txt = """\
This event will be displayed on the event list for the 'date', 'time' and 'duration' set below. A visible and, optionally, an audible alert will also be generated at the appropriate time(s) using 'RUN' and the external command 'wxremalert'. Click Help for details on the date, time and duration fields, and for date and time substitutions in the message fields. \
"""
        elif type == 's':
            title = "skeleton reminder template"
            about_txt = """\
A template for advanced users. See the remind man page. \
"""
        else:
            # unrecognized type
            raise NameError, 'unrecognized type: %s' % type 

        wx.Dialog.__init__(self, None, -1, "wxRemind: %s" % title, size=(600,400))
        self.SetBackgroundColour(nfcolor)

        # Create the text controls
        about   = wx.StaticText(self, -1, about_txt, size=(textfieldwidth+180, -1), style=wx.TE_MULTILINE)
        about.Wrap(textfieldwidth+180)

        if type == 's':
            skel_tR = wx.RadioBox(self, -1, "", wx.DefaultPosition, wx.DefaultSize,
                        skelRlist, 2, wx.RA_SPECIFY_COLS,
                        validator=TextObjectValidator(data, "skel_r"))
            self.Bind(wx.EVT_RADIOBOX, self.SetUnsavedChanges, skel_tR)
            date_t  = wx.TextCtrl(self, validator=TextObjectValidator(data, "date"), size=(textfieldwidth, -1))
            skel_tM = wx.RadioBox(self, -1, "", (20,20), wx.DefaultSize,
                        skelMlist, 4, wx.RA_SPECIFY_COLS,
                        validator=TextObjectValidator(data, "skel_m"))
            self.Bind(wx.EVT_RADIOBOX, self.SetUnsavedChanges, skel_tM)
            msg_t  = wx.TextCtrl(self, validator=TextObjectValidator(data, "msg"), size=(textfieldwidth, -1))
        else:
            date_l  = wx.StaticText(self, -1, "Date:")
            date_t  = wx.TextCtrl(self, validator=TextObjectValidator(data, "date"), size=(textfieldwidth, -1))
            msg_l  = wx.StaticText(self, -1, "Message:")
            msg_t  = wx.TextCtrl(self, validator=TextObjectValidator(data, "msg"), size=(textfieldwidth, -1))
            self.Bind(wx.EVT_TEXT, self.SetUnsavedChanges, msg_t)
            omsg_l  = wx.StaticText(self, -1, "Other Message:")
            omsg_t = wx.TextCtrl(self, validator=TextObjectValidator(data, "omsg"), size=(textfieldwidth, -1))
            self.Bind(wx.EVT_TEXT, self.SetUnsavedChanges, omsg_t)

        if type in ('t', 'a'):
            time_l  = wx.StaticText(self, -1, "Time (24 hour):")
            time_t  = wx.TextCtrl(self, validator=TextObjectValidator(data, "time"), size=(textfieldwidth, -1))
            self.Bind(wx.EVT_TEXT, self.SetUnsavedChanges, time_t)
            duration_l  = wx.StaticText(self, -1, "Duration (H:MM):")
            duration_t  = wx.TextCtrl(self, validator=TextObjectValidator(data, "dur"), size=(textfieldwidth, -1))
            self.Bind(wx.EVT_TEXT, self.SetUnsavedChanges, duration_t)

        if type == 'a':
            alertAlist = ['None', 'System sound (audible1)', 'Spoken Message (audible2)']
            alertVlist = ['None', 'Message Box', 'System Alert (visible2)']
            alert_lA  = wx.StaticText(self, -1, "Audible Alert:")
            alert_tA = wx.RadioBox(self, -1, "", wx.DefaultPosition, wx.DefaultSize,
                        alertAlist, 1, wx.RA_SPECIFY_COLS,
                        validator=TextObjectValidator(data, "alert_s"))
            self.Bind(wx.EVT_RADIOBOX, self.SetUnsavedChanges, alert_tA)
            alert_lV  = wx.StaticText(self, -1, "Visual Alert:")
            alert_tV = wx.RadioBox(self, -1, "", (20,20), wx.DefaultSize,
                        alertVlist, 1, wx.RA_SPECIFY_COLS,
                        validator=TextObjectValidator(data, "alert_d"))
            self.Bind(wx.EVT_RADIOBOX, self.SetUnsavedChanges, alert_tV)
        
        if current and reminders and current != reminders:
            filelist = [current, reminders]
            file_l = wx.StaticText(self, -1, "Reminders File:")
            file_t = wx.RadioBox(self, -1, "", wx.DefaultPosition, wx.DefaultSize,
                        filelist, 1, wx.RA_SPECIFY_COLS,
                        validator=TextObjectValidator(data, "file_s"))
            self.Bind(wx.EVT_RADIOBOX, self.SetUnsavedChanges, file_t)

        # Use standard button IDs
        help = wx.Button(self, wx.ID_HELP)
        self.Bind(wx.EVT_BUTTON, self.OnHelp, help)
        cancel = wx.Button(self, wx.ID_CANCEL)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, cancel)
        okay   = wx.Button(self, wx.ID_OK)
        okay.SetDefault()

        # Layout with sizers
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(about, 0, wx.ALL, 5)
        sizer.Add(wx.StaticLine(self), 0, wx.EXPAND|wx.ALL, 5)

        if type == 's':
            fgs = wx.FlexGridSizer(4, 1, 5, 5)
            fgs.Add(skel_tR, 0, wx.EXPAND)
            fgs.Add(date_t, 0, wx.EXPAND)
            fgs.Add(skel_tM, 0, wx.EXPAND)
            fgs.Add(msg_t, 0, wx.EXPAND)
            fgs.AddGrowableCol(1)
            if current and reminders and current != reminders:
                fgs.Add(file_l, 0, wx.ALIGN_LEFT)
                fgs.Add(file_t, -1, wx.EXPAND)
            fgs.AddGrowableCol(0)
        else:
            fgs = wx.FlexGridSizer(3, 2, 5, 5)
            fgs.Add(date_l, 0, wx.ALIGN_RIGHT)
            fgs.Add(date_t, -1, wx.EXPAND)
            fgs.Add(msg_l, 0, wx.ALIGN_RIGHT)
            fgs.Add(msg_t, -1, wx.EXPAND)
            fgs.Add(omsg_l, 0, wx.ALIGN_RIGHT)
            fgs.Add(omsg_t, -1, wx.EXPAND)
            if type in ('t', 'a'):
                fgs.Add(time_l, 0, wx.ALIGN_RIGHT)
                fgs.Add(time_t, -1, wx.EXPAND)
                fgs.Add(duration_l, 0, wx.ALIGN_RIGHT)
                fgs.Add(duration_t, -1, wx.EXPAND)
            if type == 'a':
                fgs.Add(alert_lA, 0, wx.ALIGN_RIGHT)
                fgs.Add(alert_tA, -1, wx.EXPAND)
                fgs.Add(alert_lV, 0, wx.ALIGN_RIGHT)
                fgs.Add(alert_tV, -1, wx.EXPAND)
            if current and reminders and current != reminders:
                fgs.Add(file_l, 0, wx.ALIGN_RIGHT)
                fgs.Add(file_t, -1, wx.EXPAND)
            fgs.AddGrowableCol(1)

        sizer.Add(fgs, 0, wx.EXPAND|wx.ALL, 5)

        btns = wx.StdDialogButtonSizer()
        btns.AddButton(help)
        btns.AddButton(okay)
        btns.AddButton(cancel)
        btns.Realize()
        sizer.Add(btns, 0, wx.EXPAND|wx.ALL, 5)
        self.UnsavedChanges = 0

        self.SetSizer(sizer)
        sizer.Fit(self)

    def OnCancel(self, event):
        # There will be 1 modification from pasting the selected date
        if self.UnsavedChanges > 0:
            dlg = wx.MessageDialog(self, "Abandon changes?", "Modified entries", 
                    wx.wx.NO |
                    wx.wx.YES)
            retval = dlg.ShowModal()
            dlg.Destroy()
            if retval == wx.ID_YES:
                self.data['msg'] = ''
                self.EndModal(False)
        else:
            self.EndModal(True)

    def SetUnsavedChanges(self, event):
        self.UnsavedChanges += 1

    def OnHelp(self, event):
        # show the hints page
        os.system("wxremhints &")

if __name__ == '__main__':
    import pprint
    import sys
    try:
        type = sys.argv[1]
    except:
        type = 't'
    app = wx.PySimpleApp()

    data = { "date" : "17 May 2006" }
    dlg = MyDialog(data, type)
    dlg.UnsavedChanges = -1
    dlg.ShowModal()
    dlg.Destroy()

    wx.MessageBox("You entered these values:\n\n" +
                pprint.pformat(data))

    app.MainLoop()
