import os.path, wx, wx.html
from wxremindrc import *
from version import *

msg = """<html>
<body>
<p><b>Creating new reminders</b></p>

<br> Select the target date in the calendar before pressing one of the following
keys to create the new reminder.
<center><table>
<tr><th>a</th> <td>a new timed reminder with an <em>external</em> alert.</td></tr>
<tr><th>t</th> <td>a new timed reminder with an <em>internal</em> alert.</td></tr>
<tr><th>u</th> <td>a new <em>untimed</em> reminder.</td></tr>
<tr><th>s</th> <td>a new reminder using a <em>skeleton</em> template for advanced Remind users.</td></tr>
</table></center>

<br>The selected date will automatically be placed in the date field and when
you're finished filling in the fields, you will be able to see the resulting
reminder in the event list. You can then select the new event and the status
bar will show you its details. If there's anything about it you don't like,
then you can press 'enter' to bring up the editor with the cursor at the new
reminder or 'e' to edit a 'broken' reminder that doesn't show up in the events list.

<p><b>Date Field</b></p>

<center>
<table cols="2" border="0">
<tr><th>Date String</th><th align="left">Triggers</th></tr>
<tr><th>Empty</th><td>Every day</td></tr>
<tr><th>Tue Thu</th><td>Every Tuesday and Thursday</td></tr>
<tr><th>12</th><td>The 12th of every month</td></tr>
<tr><th>Apr 2006</th><td>Every day in April, 2006</td></tr>
<tr><th>Sat 1</th><td>The 1st Saturday in every month</td></tr>
<tr><th>Sat 1 Mar</th><td>The 1st Saturday in every March</td></tr>
<tr><th>-1</th><td>The last day of every month</td></tr>
<tr><th>Wed May</th><td>Every Wednesday in every May</td></tr>
<tr><th>Tue Thu 2006</th><td>Every Tuesday and Thursday in 2006</td></tr>
<tr><th>2005</th><td>Every day in 2005</td></tr>
<tr><th>18 May 2006 *14</th><td>Every two weeks starting 18 May 2006</td></tr>
<tr><th>Fri UNTIL 1 Jul 2006</th><td>Every Friday until 1 July 2006</td></tr>
<tr><th>5 May 2006 *1 +2 UNTIL 15 May 2006</th><td>Every day from May 5 until May 15 plus 2 days advance notice.</td></tr>
</table>
</center>


<p><b>Time Field</b></p>
<br>

Must begin with a 24 hour time in the format HH:MM. Can optionally be followed
by + and * strings.

<center>
<table cols="2" border="0">
<tr><th>Time String</th><th align="left">Triggers</th></tr>
<tr><th>13:00</th><td>1:00PM</td></tr>
<tr><th>10:00 +45 *20</th><td>9:15AM, 9:35AM, 9:55AM and 10:00AM</td></tr>
</table>
</center>
</p>

<p><b>Duration Field</b></p>
<br>
Must specify hours and minutes in the format H:MM.
<center>
<table cols="2" border="0">
<tr><th>Duration String</th><th align="left">Ends at start time plus</th></tr>
<tr><th>1:15</th><td>75 minutes</td></tr>
<tr><th>0:15</th><td>15 minutes</td></tr>
</table>
</center>

<p><b>Message / Other Message Fields</b></p>
<br>
'Message' is a required field for all types of reminders. 'Other Message' is
optional and, if present, will not appear in wxRemind's event list or other
remind "calendar" uses, but will appear in normal, non-calendar uses and thus
in alerts. For example,

<center>
<table>
<tr><td>Message: Dentist appointment</td></tr>
<tr><td>Other Message: %1</td></tr>
</table>
</center>

<br>would appear on the relevant event list as <em>Dentist appointment</em> but
the message for an alert triggered 20 minutes before the appointment would be
<em>Dentist appointment 20 minutes from now</em>. See below for the use of %1
and other time and date substitution strings. Technically, if 'Other Message'
were empty, the the reminder message would simply be <em>Dentist
appointment%</em>.  The addition of the 'Other Message', on the other hand,
yields the message <em>%"Dentist appointment%" %1%</em>.


<p><b>Message Fields Date Substitution Strings</b></p>

<br>
<center>
<table cols="2" border="0">
<tr><th>String</th><th align="left">Replaced by</th></tr>
<tr><th>%a</th><td>"on weekday, day month year", "tomorrow", "today", etc.</td></tr>
<tr><th>%b</th><td>"in n days' time", "tomorrow", "today", etc.</td></tr>
</table>
</center>
<p>Also %c ... %z &mdash; see the remind man page.</p>


<p><b>Message Fields Time Substitution Strings</b></p>

<br>
<center>
<table cols="2" border="0">
<tr><th>String</th><th align="left">Replaced by</th></tr>
<tr><th>%1</th><td>"now", "m minutes from now", "m minutes ago", etc.</td></tr>
<tr><th>%2</th><td>"at hh:mmAM" or "at hh:mmPM"</td></tr>
<tr><th>%3</th><td>"at hh:mm" in 24 hour format</td></tr>
</table>
</center>

<p>Also %4 ... %0 &mdash; see the remind man page.</p>


<p><b>Floating Reminders (Optional)</b></p>

<br>Requires including wxremfloat in your reminders file prior to use.
<br>Create an untimed reminder with, e.g., '[float(2006,Apr,15,10)]' in the date field (don't forget the brackets) and 'File Taxes' in the msg field. The resulting reminder would then be: 
<br>REM [float(2006,Apr,15,10)] MSG File Taxes%
<br>and this would display as follows:
<center>
<table cols="5" border="0">
<tr><th>Today</th>
<th align="left">Calendar for</th><th>Display</th>
<th align="left">Calendar for</th><th>Display</th>
</tr>
<tr>
<th>Apr 2</th>
<th>Apr 5</th><td>File Taxes (in 10 days)</td>
<th>Apr 15</th><td>File Taxes (today)</td>
</tr>
<tr>
<th>Apr 6</th>
<th>Apr 6</th><td>File Taxes (in 9 days)</td>
<th>Apr 15</th><td>File Taxes (today)</td>
</tr>
<tr>
<th>Apr 14</th>
<th>Apr 14</th><td>File Taxes (tomorrow)</td>
<th>Apr 15</th><td>File Taxes (today)</td>
</tr>
<tr>
<th>Apr 16</th>
<th>Apr 16</th><td>File Taxes (yesterday)</td>
<th>Apr 15</th><td>File Taxes (today)</td>
</tr>
</table>
</center>
<br>The floating reminder will continue to remind (nag) until it is deleted.
<p>Consider using yeaGTD (href://www.duke.edu/~dgraham/yeaGTD) to manage your todo's.It's design goal is to be for todo's what remind is for reminders.</p>


<p><b>Timed and Alert Reminders</b></p>

<br>

Both timed and alert reminders generate popup and/or audible alerts. For <em>timed</em> reminders these are generated <em>internally</em> by wxRemind running in zombie (-z) mode using standard 'MSG' remind entries. These internal alerts require that wxRemind is running and provide options to cancel further reminders and to control how long the alerts are displayed before being automatically closed.

<br>

Alert reminders also provide popup and/or audible alerts but these are generated <em>externally</em> using 'RUN' rather than 'MSG' in the reminder to invoke Remind (running in the background) to call wxremalert. These alerts will be triggered if Remind is running <em>whether or not wxRemind is running</em>, but do not support the cancelation or automatic closing features of the internal, timed reminders. 

<br>
For example, with the following settings in ~/.wxremindrc

<table>
<tr><td>&nbsp;&nbsp;alert_display = 1</td></tr>
<tr><td>&nbsp;&nbsp;alert_sound = 2</td></tr>
<tr><td>&nbsp;&nbsp;alert_greeting = 1</td></tr>
<tr><td>&nbsp;&nbsp;alert_whom = 'Dan'</td></tr> 
<tr><td>&nbsp;&nbsp;alert_parsenums = 1</td></tr>
<tr><td>&nbsp;&nbsp;alert_other_message = '%1'</td></tr>
</table>
a new alert reminder created with April 11, 2006 selected on the calendar
would initially have the following entries:
<table>
<tr><td>&nbsp;&nbsp;Date: 11 Apr 2006</td></tr>
<tr><td>&nbsp;&nbsp;Time:</td></tr>
<tr><td>&nbsp;&nbsp;Duration:</td></tr>
<tr><td>&nbsp;&nbsp;Message:</td></tr>
<tr><td>&nbsp;&nbsp;Other Message: %1</td></tr>
<tr><td>&nbsp;&nbsp;Visual Alert: 2nd button (Pop-up Display) checked</td></tr>
<tr><td>&nbsp;&nbsp;Audible Alert: 3rd button (Spoken Message) checked</td></tr>
</table>
After entering '10:50 +5' for Time, '1:15' for Duration and 'Economics 201'
for Message, the resulting reminder would be
<table>
<tr><td>REM 11 Apr 2006 AT 10:50 +5 DUR 1:15 RUN wxremalert -d1 -s2 %"Economics 201%" %1%</td></tr>
</table>
The calendar for Apr 11 would then display
<table>
<tr><td>10:50AM - 12:05PM  Economics 201</td></tr>
</table>
and, at 10:45 on Apr 11 the following message would be spoken
<table>
<tr><td>"Good morning, Dan. The time is ten forty-five. Economics two oh one five
minutes from now."</td></tr>
</table>
and later, at 10:50:
<table>
<tr><td>"Good morning, Dan. The time is ten fifty. Economics two oh one now."</td></tr>
</table>
</body>
</html>
"""

class Hints(wx.Frame):
    def __init__(self, parent=None):
        wx.Frame.__init__(self, parent, -1, 
                'wxRemind: Hints for creating reminders', 
                size=(620, 700))
        html = wx.html.HtmlWindow(self)
        html.SetPage(msg)
        button = wx.Button(self, wx.ID_OK, "OK")
        button.SetDefault()
        self.Bind(wx.EVT_BUTTON, self.OnQuit, button)
        self.Bind(wx.EVT_CHAR, self.OnChar)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(html, 1, wx.EXPAND | wx.ALL, 4)
        sizer.Add(button, 0, wx.ALIGN_CENTER | wx.BOTTOM, 4)
        self.SetSizer(sizer)
        self.Layout()

    def OnChar(self, event):
        keycode = event.GetKeyCode()
        if keycode == 17:         # Ctrl-Q quit
            self.OnQuit(event)
        else:
            event.Skip()

    def OnQuit(self,event):
        self.Destroy()
            
class App(wx.App):
    def OnInit(self):
        self.frame = Hints()
        self.frame.Show(True)
        self.SetTopWindow(self.frame)
        return True
        
def main():
    app = App()
    app.MainLoop()
    
if __name__ == '__main__':
    main()
