#!/usr/bin/env python
"""
wxremalert.py

Copyright (c) 2006-2007 Daniel Graham <daniel.graham@duke.edu>. All rights reserved.

License:
This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version. [ http://www.gnu.org/licenses/gpl.html ]

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
"""

import wx
import sys, os, os.path, datetime, re

from version import *
from wxremindrc import *
import wx.html

# class Alert(wx.Frame):
class Alert(wx.Dialog):
    def __init__(self, parent, seconds, again, tag, message, ignore):
        # wx.Frame.__init__(self, parent, -1, 'wxRemind Alert', size=alertsize)
        wx.Dialog.__init__(self, parent, -1, 'wxRemind Alert', 
            size=alertsize, style=wx.DEFAULT_DIALOG_STYLE | wx.STAY_ON_TOP)
        html = wx.html.HtmlWindow(self)
        html.SetBackgroundColour(fcolor)
        text = '''<html>
        <body>
        <font size="%s">
        <center><b><font color="%s">%s</font></b></center>
        </font>
        </body>
        </html> 
        ''' % (alertfontsize, alertcolor, message)
        html.SetPage(text)
        self.tag = tag
        self.ignore = ignore
        box = wx.BoxSizer(wx.VERTICAL)
        box.Add(html, 1, wx.EXPAND | wx.ALL, 5)
        subbox = wx.BoxSizer(wx.HORIZONTAL)

        if again:
            lab  = wx.StaticText(self, -1, 
                "Cancel subsequent repetitions of this reminder?")
            subbox.Add(lab,0,wx.ALIGN_RIGHT | wx.ALL, 4)
            btn = wx.Button(self, -1, "Yes")
            self.Bind(wx.EVT_BUTTON, self.OnCancel, btn)
            subbox.Add(btn, 0, wx.ALIGN_RIGHT | wx.ALL, 4)
            btn = wx.Button(self, wx.ID_OK, "No")
        else:
            btn = wx.Button(self, wx.ID_OK, "OK")
        btn.SetDefault()
        subbox.Add(btn, 0, wx.ALIGN_RIGHT | wx.ALL, 4)
        self.Bind(wx.EVT_BUTTON, self.OnClose, btn)
        box.Add(subbox, 0, wx.ALIGN_RIGHT)
        self.SetSizer(box)
        # self.SetAutoLayout(True)
        self.Layout()

        if alert_sound == 1:
            os.system(audible1)

        elif alert_sound == 2:
            spoken_msg = spoken_message(message)
            os.system(audible2 % spoken_msg)

        if seconds > 0:
            self.timeout = seconds*1000
            self.timer = wx.CallLater(self.timeout, self.Notify)

    def Notify(self):
        # Now that the timer has expired, destroy the dialog
        self.Destroy()

    def OnCancel(self, event):
        self.ignore.append(self.tag)
        self.Destroy()

    def OnClose(self, event):
        self.Destroy()


def spoken_message(message):
    spoken_msg = message
    if message and alert_parsenums:
        # look for three consecutive digits preceeded by a non-digit and
        # followed by either a word boundary or a non-digit
        regex= re.compile(r'\D?(\d{3,3})[\b\D]?')
        nums = re.findall(regex, spoken_msg)
        if len(nums) > 0:
            for num in nums:
                d0 = num[0]
                d1 = num[1]
                d2 = num[2]
                if d1 == '0' and d2 == '0':
                    # replace the two zeros with 'hundred'
                    rep = "%s hundred" % (d0)
                elif d1 == '0':
                    # replace the zero with an uppercase o
                    d1 = 'O'
                    # separate the leading digit from the following 2 digits
                    rep = " %s %s%s" % (d0,d1,d2)
                else:
                    rep = " %s %s%s" % (d0,d1,d2)
                    # replace the original number
            spoken_msg = re.sub(num, rep, spoken_msg)
            spoken_msg = re.sub('@', 'at', spoken_msg)
            spoken_msg.strip()

    hours,minutes,seconds = \
            datetime.datetime.today().strftime("%H %M %S").split()

    hours = int(hours)
    minutes = int(minutes)
    seconds = int(seconds)

    if twelvehour:
        hr = hour12
        hrs = hours12
        nullhrs = ""
        onehr = onehour12
        nullmin =  nullminute12
        underten = undertenminuteprefix12
        min = minute12
        mins = minutes12
        spokenhours = [12, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12,
            1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
    else:
        hr = hour24
        hrs = hours24
        nullhrs = nullhours24
        onehr = onehour24
        nullmin =  nullminute24
        underten = undertenminuteprefix24
        min = minute24
        mins = minutes24
        spokenhours = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12,
            13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23]

    # round current time up
    if seconds >= 50:
        minutes += 1
        if minutes >= 60:
            minutes = 0
            hours += 1
            if hours >= 24:
                hours = 0

    if spokenhours[hours] == 0:
        time = "%s" % nullhrs
    elif spokenhours[hours] == 1:
        time = "%s" % (onehr)
    else:
        time = "%s" % spokenhours[hours]
        if hrs:
            time += " %s" % hrs


    if minutes == 0:
        minutestr = nullmin
    else:
        if minutes <= 9 and underten:
            minutestr = "%s %d" % (underten, minutes)
        else:
            minutestr = "%d" % (minutes)
        if min and minutes == 1: 
            minutestr += " %s" % min
        elif mins and minutes > 1:
            minutestr += " %s" % mins

    if minutestr:
        time += " %s" % minutestr

    message = ""

    if alert_greeting:
        try:
            Who = alert_whom
        except:
            Who = ''
        if hours < 12:
            message = "%s %s. " % (goodmorning, Who)
        elif hours < 18:
            message = "%s %s. " % (goodafternoon, Who)
        else:
            message = "%s %s. " % (goodevening, Who)

    message += "%s %s. %s" % (thetimeis, time, spoken_msg)
    return message

def alert(sound, display, message):
    global audible1, audible2, visible2, alert_sound, alert_display
    if sound == None: 
        alert_sound = int(alert_sound)
    else:
        alert_sound = int(sound)
    if display == None: 
        alert_display = int(alert_display)
    else:
        alert_display = int(display)
    event_msg = spoken_msg = message
    if alert_sound == 0 and alert_display == 0:
        sys.exit(0)

    if alert_sound == 1:
        os.system(audible1)

    elif alert_sound == 2:
        spoken_msg = spoken_message(message)
        os.system(audible2 % spoken_msg)
        # print "%s" % message

    if alert_display == 1:
        # we're running this externally and don't have tag information
        # so we'll just show a dialog box
        app = wx.PySimpleApp()
        dlg = wx.MessageDialog(None, "%s" % event_msg,
                              'wxRemind alert',
                              wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    elif alert_display == 2:
        # call growlnotify
        os.system(visible2 % event_msg)

def main():
    from optparse import OptionParser
    import os.path
    global options, args
    parser = OptionParser(version = "wxremalert %s" % version,
        usage = "%prog [options] message",
        description = """Produce an audible and/or visible alert using settings from ~/.wxremindrc.  Designed to be called from remind's 'RUN'.""")
    parser.add_option("-d", "--display", action = "store", type="choice", 
        dest='display', choices = ["0", "1", "2"], default = None,
        help = """0: none; 1: message box; 2: visable2""")
    parser.add_option("-s", "--sound", action = "store", type="choice", 
        dest='sound', choices = ["0", "1", "2"], default = None,
        help = """0: none; 1: audible1; 2: audible2""")

    (options, args) = parser.parse_args()
    if len(args):
        message = ' '.join(args)
    else:
        message = ''
    alert(options.sound, options.display, message)
